/***
 * Sangjog CLI
 * Copyright(c) 2022 Hirak Santra
 */
'use strict';
const { spawn, exec } = require('child_process');
const psTree = require('ps-tree');

/***
 * Child Process Manager class
 */
class CPManager {

  static KILL_SIGNAL = 15;

  static runCommand(CMD, inputOptions, callBacksObj) {
      const options = {
          stdio: 'inherit',
          shell: true,
          cwd: process.cwd(),
          ...inputOptions
      };
      const childProcess = spawn(CMD, options);
      childProcess.on('error', (err) => {
          console.error(`SANGJOG_ERROR: ${err}`);
      });
      if(childProcess && callBacksObj) {
          const { close, stderr } = callBacksObj;
          if(close) {
              childProcess.on('close', close);
          }
      }
      return childProcess;
  }

  static kill(pid, callback) {
      const osType = process.platform;
      if(osType==='win32') {
          exec(`taskkill /pid ${pid} /T /F`, { windowsHide: true }, callback);
      } else if(osType==='darwin') {
          exec(`kill -${CPManager.KILL_SIGNAL} ${pid}`, callback);
      } else {
          psTree(pid, function(err, kids) {
             spawn('kill', [`-${CPManager.KILL_SIGNAL}`, pid].concat( kids.map(p => p.PID) )).on('close', callback);
          });
      }
  }

}

module.exports = CPManager;
