/***
 * Sangjog CLI
 * Copyright(c) 2022 Hirak Santra
 */
 'use strict';

const nodePath = require('path');
const SangjogZipExtractor = require('./sangjog-zip-extractor');
const Utils = require('./utils');
const AppRunner = require('./app-generators/app-runner');

const Conf = require('./sangjog.json');
const F = {
    cr: 'create',
    gn: 'generate',
    prj: 'project',
    cnt: 'controller',
    srv: 'service',
    i: 'install',
    s: 'start',
    d: 'dev',
    b: 'build'
}

class SangjogCli {

    cmd = [];

    currentDir = process.cwd();

    clear() {
        this.cmd = [];
    }

    consume(cmd) {
        this.cmd = cmd;
    }

    run() {
        const MAIN_CMD = this.cmd[0];
        this.cmd.shift();
        switch(MAIN_CMD) {
            case F.cr:
                this.create();
                break;
            case F.i:
                AppRunner.install();
                break;
            case F.d:
                AppRunner.develop();
                break;
            case F.s:
                AppRunner.start();
                break;
            case F.b:
                AppRunner.build();
                break;
            case F.gn:
                this.generate();
                break;
            default:
                Utils.throwArgErr();
        }
    }

    create() {
        if(this.cmd[0]===F.prj && this.cmd.length===2) {
            this.createProject(this.cmd[1]);
        } else {
            Utils.throwArgErr();
        }
    }

    createProject(projName) {
        const name = Utils.getFolderName(projName);
        console.log(`Please wait while creating the Sangjog Project....`);
        SangjogZipExtractor.downloadFile(Conf.project.gitZipUrl, {
            inWhichDir: this.currentDir,
            zipFolderName: Conf.project.zipFolderName,
            devProjectName: name,
        }, ()=>{
            SangjogZipExtractor.modifyPackageJson({projName, name, currentDir: this.currentDir});
            console.log(`Congratulation! One Sangjog Project has been created successfully....`);
        });
    }

    //TBI
    generate() {

    }

    //TBI
    generateController() {

    }

    //TBI
    generateService() {

    }

}

module.exports = new SangjogCli();